"""The pytest-celery plugin provides a set of built-in components called
:ref:`vendors`.

This module is part of the Redis Backend vendor.
"""

CELERY_REDIS_BACKEND = "celery_redis_backend"
DEFAULT_REDIS_BACKEND = "default_redis_backend"
