"""The pytest-celery plugin provides a set of built-in components called
:ref:`vendors`.

This module is part of the Redis Broker vendor.
"""

CELERY_REDIS_BROKER = "celery_redis_broker"
DEFAULT_REDIS_BROKER = "default_redis_broker"
